@@ -23,7 +23,6 @@ import Data.Text (Text)
 import Data.Time (TimeZone, UTCTime)
 import Graphics.Vty.Image
 import System.IO (Handle)
-import qualified Config
 import qualified Data.ByteString as B
 import qualified Data.Map as Map
 import qualified Data.Set as Set
@@ -76,7 +75,6 @@ data ClientState = ClientState
   , _clientAutomation :: [EventHandler]
   , _clientTimers     :: Map UTCTime [TimerEvent]
   , _clientTimeZone   :: TimeZone
-  , _clientConfig     :: Config.Value
   }
   -- TODO: split this record into logical pieces
 
