@@ -188,6 +188,12 @@ initialServerSettings !args =
                                let p = maybe defaultSocksPort fromIntegral
                                      $ defaultNum hostTxt "socks-port" args
                                return (h,p)
+
+       , _ssServerCerts   = toListOf
+                              ( cmdArgConfigValue
+                              . configPath hostTxt "server-certificates"
+                              . values . text . unpacked
+                              ) args
        }
 
 
