@@ -185,7 +185,8 @@ initializeConnection pass nick user real h =
 
 driver :: Vty -> TChan Event -> TChan (UTCTime, MsgFromServer) -> ClientState -> IO ()
 driver vty vtyEventChan ircMsgChan st0 =
-  do let (scroll', pic) = picForState st0
+  do now <- getCurrentTime
+     let (scroll', pic) = picForState now st0
          st1 = set clientScrollPos scroll' st0
      update vty pic
      e <- readEitherTChan vtyEventChan ircMsgChan
@@ -207,9 +208,9 @@ driver vty vtyEventChan ircMsgChan st0 =
   processTimerEvent e st =
     case e of
       TransmitPing ->
-        do sendTimestampPing st
-           st' <- schedulePing st
-           considerTimers st'
+        do st1 <- sendTimestampPing st
+           st2 <- schedulePing st1
+           considerTimers st2
 
       DropOperator chan ->
         do clientSend (modeCmd chan ["-o",views connNick idBytes conn]) st
@@ -285,7 +286,9 @@ inputLogic st =
     '/':_ -> case commandEvent st of
                (img, Nothing) -> (img, return (KeepGoing st))
                (img, Just m ) -> (img, m)
-    txt -> (stringWithControls defAttr txt, fmap KeepGoing (doSendMessageCurrent SendPriv st))
+    txt -> ( ircFormattedText' explicitFormatting (Text.pack txt)
+           , fmap KeepGoing (doSendMessageCurrent SendPriv st)
+           )
 
 keyEvent :: Key -> [Modifier] -> ClientState -> IO KeyEventResult
 keyEvent k ms st =
@@ -355,6 +358,18 @@ doSendMessageCurrent sendType st =
       doSendMessage sendType c (Text.pack (clientInput st)) st
     _ -> return st
 
+-- Pessimistic maximum length computation for the message part for
+-- privmsg and notice. Note that the need for the limit is because
+-- the server will limit the length of the message sent out to each
+-- client, not just the length of the messages it will recieve.
+computeMaxMessageLength :: Identifier -> String -> Identifier -> Int
+computeMaxMessageLength nick user target
+  = 512
+  - 63 -- max hostname
+  - length (":~!@ PRIVMSG :\r\n"::String)
+  - B8.length (idBytes nick)
+  - length user
+  - B8.length (idBytes target)
 
 doSendMessage :: SendType -> Identifier -> Text -> ClientState -> IO ClientState
 
@@ -363,21 +378,28 @@ doSendMessage sendType _ message st
 
 doSendMessage sendType target message st =
   do let bs = case sendType of
-                SendPriv -> privMsgCmd target (Text.encodeUtf8 message)
-                SendAction -> privMsgCmd target ("\SOHACTION " <>
-                                                 Text.encodeUtf8 message <> "\SOH")
-                SendNotice -> noticeCmd target (Text.encodeUtf8 message)
-                SendCtcp cmd -> ctcpRequestCmd target
+                SendPriv -> map (privMsgCmd target . Text.encodeUtf8) messages
+                                -- long actions aren't split up
+                SendAction -> [privMsgCmd target ("\SOHACTION " <>
+                                                 Text.encodeUtf8 message <> "\SOH")]
+                SendNotice -> map (noticeCmd target . Text.encodeUtf8) messages
+                SendCtcp cmd -> [ctcpRequestCmd target
                                  (Text.encodeUtf8 (Text.pack (map toUpper cmd)))
-                                 (Text.encodeUtf8 message)
-     clientSend bs st
+                                 (Text.encodeUtf8 message)]
+     mapM_ (`clientSend` st) bs
      now <- getCurrentTime
-     let myNick = view connNick conn
-         myModes = view (connChannels . ix target' . chanUsers . ix myNick) conn
+     let myModes = view (connChannels . ix target' . chanUsers . ix myNick) conn
      return (addMessage target' (fakeMsg now myModes) (clearInput st))
 
   where
-  conn = view (clientServer0 . ccConnection) st
+  myNick = view connNick conn
+
+  maxMessageLength =
+    computeMaxMessageLength myNick (view (ccServerSettings.ssUser) ircconn) target
+  messages = Text.chunksOf maxMessageLength message
+
+  ircconn = view clientServer0 st
+  conn = view ccConnection ircconn
 
   (statusmsg, target') = splitStatusMsg target conn
 
@@ -587,14 +609,14 @@ commandEvent st = commandsParser (clientInput st)
   , ("reconnect", [], pure (doReconnect st'))
 
   , ("ping", [],
-    (\msg -> st' <$ doPingCmd msg st')
+    (\msg -> doPingCmd msg st')
     <$> pRemainingNoSp)
 
   ]
 
-doPingCmd :: String -> ClientState -> IO ()
+doPingCmd :: String -> ClientState -> IO ClientState
 doPingCmd ""  st = sendTimestampPing st
-doPingCmd msg st = clientSend (pingCmd (Text.encodeUtf8 (Text.pack msg))) st
+doPingCmd msg st = st <$ clientSend (pingCmd (Text.encodeUtf8 (Text.pack msg))) st
 
 doReconnect :: ClientState -> IO ClientState
 doReconnect st =
@@ -737,8 +759,8 @@ doQuote cmd st = st <$ clientSend (Text.encodeUtf8 (Text.pack (cmd ++ "\r\n")))
 -- Primary UI rendering
 ------------------------------------------------------------------------
 
-picForState :: ClientState -> (Int,Picture)
-picForState st = (scroll', pic)
+picForState :: UTCTime -> ClientState -> (Int,Picture)
+picForState now st = (scroll', pic)
   where
   conn = view (clientServer0 . ccConnection) st
 
@@ -760,7 +782,7 @@ picForState st = (scroll', pic)
     , mainFocusImage
     ]
 
-  divider = dividerImage st
+  divider = dividerImage now st
 
   -- Pad the main image when it doesn't fill the screen
   -- so that it starts at the bottom of the frame
@@ -841,25 +863,28 @@ textbox st
   beginning = char (withForeColor defAttr brightBlack) '^'
   ending    = char (withForeColor defAttr brightBlack) '$'
 
-dividerImage :: ClientState -> Image
-dividerImage st
+dividerImage :: UTCTime -> ClientState -> Image
+dividerImage now st
   = extendToWidth (nickPart <|> channelPart <|> lagPart)
   where
   conn = view (clientServer0 . ccConnection) st
 
   myNick = view connNick conn
 
-  lagPart =
-    case view connPingTime conn of
-      Just s ->
+  drawLagTime color s =
        let s' = realToFrac s :: Centi -- truncate to two decimal places
            sStr = showFixed True s' <> "s"
        in
                 string defAttr "-["
-            <|> string (withForeColor defAttr yellow) sStr
+            <|> string (withForeColor defAttr color) sStr
             <|> string defAttr "]"
 
-      _ -> emptyImage
+  lagPart =
+    case view connPingTime conn of
+      NoPing        -> emptyImage
+      PingSent when -> drawLagTime red (realToFrac (diffUTCTime now when))
+      PingTime s    -> drawLagTime yellow s
+
 
   channelPart =
     ifoldr (\i x xs -> drawOne i x <|> xs)
@@ -1050,10 +1075,11 @@ schedulePing st =
      let pingTime = addUTCTime 60 now
      return (addTimerEvent pingTime TransmitPing st)
 
-sendTimestampPing :: ClientState -> IO ()
+sendTimestampPing :: ClientState -> IO ClientState
 sendTimestampPing st =
   do now <- getCurrentTime
      let ts = realToFrac (utcTimeToPOSIXSeconds now) :: Pico
          chopTrailingZeros = True
          tsStr = B8.pack (showFixed chopTrailingZeros ts)
      clientSend (pingCmd tsStr) st
+     return $! set (clientServer0.ccConnection.connPingTime) (PingSent now) st
